@@ -2,6 +2,7 @@
 # Copyright (C) 2007-2008 Barry Pederson <bp@barryp.org>
 from __future__ import absolute_import, unicode_literals
 
+from .serialization import GenericContent
 # Intended to fix #85: ImportError: cannot import name spec
 # Encountered on python 2.7.3
 # "The submodules often need to refer to each other. For example, the
@@ -12,7 +13,6 @@ from __future__ import absolute_import, unicode_literals
 # Source:
 #   http://stackoverflow.com/a/14216937/4982251
 from .spec import Basic
-from .serialization import GenericContent
 
 __all__ = ['Message']
 
@@ -55,7 +55,7 @@ class Message(GenericContent):
         message_id: shortstr
             The application message identifier
 
-        timestamp: datetime.datetime
+        timestamp: unsigned long
             The message timestamp
 
         type: shortstr
@@ -102,11 +102,10 @@ class Message(GenericContent):
         ('cluster_id', 's')
     ]
 
-    #: set by basic_consume/basic_get
-    delivery_info = None
-
     def __init__(self, body='', children=None, channel=None, **properties):
         super(Message, self).__init__(**properties)
+        #: set by basic_consume/basic_get
+        self.delivery_info = None
         self.body = body
         self.channel = channel
 
