@@ -5,13 +5,12 @@ from __future__ import absolute_import, unicode_literals
 import logging
 import socket
 from collections import defaultdict
-from warnings import warn
 
 from vine import ensure_promise
 
 from . import spec
 from .abstract_channel import AbstractChannel
-from .exceptions import (ChannelError, ConsumerCancelled,
+from .exceptions import (ChannelError, ConsumerCancelled, MessageNacked,
                          RecoverableChannelError, RecoverableConnectionError,
                          error_for_code)
 from .five import Queue
@@ -21,11 +20,6 @@ __all__ = ['Channel']
 
 AMQP_LOGGER = logging.getLogger('amqp')
 
-EXCHANGE_AUTODELETE_DEPRECATED = """\
-The auto_delete flag for exchanges has been deprecated and will be removed
-from py-amqp v1.5.0.\
-"""
-
 REJECTED_MESSAGE_WITHOUT_CALLBACK = """\
 Rejecting message with delivery tag %r for reason of having no callbacks.
 consumer_tag=%r exchange=%r routing_key=%r.\
@@ -93,9 +87,14 @@ class Channel(AbstractChannel):
         spec.method(spec.Tx.SelectOk),
         spec.method(spec.Confirm.SelectOk),
         spec.method(spec.Basic.Ack, 'Lb'),
+        spec.method(spec.Basic.Nack, 'Lb'),
     }
     _METHODS = {m.method_sig: m for m in _METHODS}
 
+    _ALLOWED_METHODS_WHEN_CLOSING = (
+        spec.Channel.Close, spec.Channel.CloseOk
+    )
+
     def __init__(self, connection,
                  channel_id=None, auto_decode=True, on_open=None):
         if channel_id:
@@ -138,6 +137,7 @@ class Channel(AbstractChannel):
             spec.Basic.Deliver: self._on_basic_deliver,
             spec.Basic.Return: self._on_basic_return,
             spec.Basic.Ack: self._on_basic_ack,
+            spec.Basic.Nack: self._on_basic_nack,
         })
 
     def collect(self):
@@ -217,12 +217,14 @@ class Channel(AbstractChannel):
             if is_closed:
                 return
 
+            self.is_closing = True
             return self.send_method(
                 spec.Channel.Close, argsig,
                 (reply_code, reply_text, method_sig[0], method_sig[1]),
                 wait=spec.Channel.CloseOk,
             )
         finally:
+            self.is_closing = False
             self.connection = None
 
     def _on_close(self, reply_code, reply_text, class_id, method_id):
@@ -272,10 +274,11 @@ class Channel(AbstractChannel):
                 is the ID of the method.
         """
         self.send_method(spec.Channel.CloseOk)
-        self._do_revive()
-        raise error_for_code(
-            reply_code, reply_text, (class_id, method_id), ChannelError,
-        )
+        if not self.connection.is_closing:
+            self._do_revive()
+            raise error_for_code(
+                reply_code, reply_text, (class_id, method_id), ChannelError,
+            )
 
     def _on_close_ok(self):
         """Confirm a channel close.
@@ -604,9 +607,6 @@ class Channel(AbstractChannel):
                 implementation.  This field is ignored if passive is
                 True.
         """
-        if auto_delete:
-            warn(VDeprecationWarning(EXCHANGE_AUTODELETE_DEPRECATED))
-
         self.send_method(
             spec.Exchange.Declare, argsig,
             (0, exchange, type, passive, durable, auto_delete,
@@ -1135,8 +1135,8 @@ class Channel(AbstractChannel):
                 True.
 
         Returns a tuple containing 3 items:
-            the name of the queue (essential for automatically-named queues)
-            message count
+            the name of the queue (essential for automatically-named queues),
+            message count and
             consumer count
         """
         self.send_method(
@@ -1215,6 +1215,8 @@ class Channel(AbstractChannel):
                 client should not wait for a reply method.  If the
                 server could not complete the method it will raise a
                 channel or connection exception.
+
+        If nowait is False, returns the number of deleted messages.
         """
         return self.send_method(
             spec.Queue.Delete, argsig,
@@ -1275,7 +1277,7 @@ class Channel(AbstractChannel):
                 server could not complete the method it will raise a
                 channel or connection exception.
 
-        if nowait is False, returns a message_count
+        If nowait is False, returns a number of purged messages.
         """
         return self.send_method(
             spec.Queue.Purge, argsig, (0, queue, nowait),
@@ -1557,14 +1559,26 @@ class Channel(AbstractChannel):
         """
         p = self.send_method(
             spec.Basic.Consume, argsig,
-            (0, queue, consumer_tag, no_local, no_ack, exclusive,
-             nowait, arguments),
+            (
+                0, queue, consumer_tag, no_local, no_ack, exclusive,
+                nowait, arguments
+            ),
             wait=None if nowait else spec.Basic.ConsumeOk,
+            returns_tuple=True
         )
 
-        # XXX Fix this hack
-        if not nowait and not consumer_tag:
-            consumer_tag = p
+        if not nowait:
+            # send_method() returns (consumer_tag,) tuple.
+            # consumer_tag is returned by broker using following rules:
+            # * consumer_tag is not specified by client, random one
+            #   is generated by Broker
+            # * consumer_tag is provided by client, the same one
+            #   is returned by broker
+            consumer_tag = p[0]
+        elif nowait and not consumer_tag:
+            raise ValueError(
+                'Consumer tag must be specified when nowait is True'
+            )
 
         self.callbacks[consumer_tag] = callback
 
@@ -1572,7 +1586,11 @@ class Channel(AbstractChannel):
             self.cancel_callbacks[consumer_tag] = on_cancel
         if no_ack:
             self.no_ack_consumers.add(consumer_tag)
-        return p
+
+        if not nowait:
+            return consumer_tag
+        else:
+            return p
 
     def _on_basic_deliver(self, consumer_tag, delivery_tag, redelivered,
                           exchange, routing_key, msg):
@@ -1630,7 +1648,8 @@ class Channel(AbstractChannel):
                 reliability.  Messages can get lost if a client dies
                 before it can deliver them to the application.
 
-        Non-blocking, returns a message object, or None.
+        Non-blocking, returns a amqp.basic_message.Message object,
+        or None if queue is empty.
         """
         ret = self.send_method(
             spec.Basic.Get, argsig, (0, queue, no_ack),
@@ -1665,6 +1684,11 @@ class Channel(AbstractChannel):
         configuration and distributed to any active consumers when the
         transaction, if any, is committed.
 
+        When channel is in confirm mode (when Connection parameter
+        confirm_publish is set to True), each message is confirmed. When
+        broker rejects published message (e.g. due internal broker
+        constrains), MessageNacked exception is raised.
+
         PARAMETERS:
             exchange: shortstr
 
@@ -1725,6 +1749,16 @@ class Channel(AbstractChannel):
         if not self.connection:
             raise RecoverableConnectionError(
                 'basic_publish: connection closed')
+
+        capabilities = self.connection.\
+            client_properties.get('capabilities', {})
+        if capabilities.get('connection.blocked', False):
+            try:
+                # Check if an event was sent, such as the out of memory message
+                self.connection.drain_events(timeout=0)
+            except socket.timeout:
+                pass
+
         try:
             with self.connection.transport.having_timeout(timeout):
                 return self.send_method(
@@ -1736,11 +1770,18 @@ class Channel(AbstractChannel):
     basic_publish = _basic_publish
 
     def basic_publish_confirm(self, *args, **kwargs):
+
+        def confirm_handler(method, *args):
+            # When RMQ nacks message we are raising MessageNacked exception
+            if method == spec.Basic.Nack:
+                raise MessageNacked()
+
         if not self._confirm_selected:
             self._confirm_selected = True
             self.confirm_select()
         ret = self._basic_publish(*args, **kwargs)
-        self.wait(spec.Basic.Ack)
+        # Waiting for confirmation of message.
+        self.wait([spec.Basic.Ack, spec.Basic.Nack], callback=confirm_handler)
         return ret
 
     def basic_qos(self, prefetch_size, prefetch_count, a_global,
@@ -1801,11 +1842,16 @@ class Channel(AbstractChannel):
 
             a_global: boolean
 
-                apply to entire connection
+                Defines a scope of QoS. Semantics of this parameter differs
+                between AMQP 0-9-1 standard and RabbitMQ broker:
 
-                By default the QoS settings apply to the current
-                channel only.  If this field is set, they are applied
-                to the entire connection.
+                MEANING IN AMQP 0-9-1:
+                    False:  shared across all consumers on the channel
+                    True:   shared across all consumers on the connection
+                MEANING IN RABBITMQ:
+                    False:  applied separately to each new consumer
+                            on the channel
+                    True:   shared across all consumers on the channel
         """
         return self.send_method(
             spec.Basic.Qos, argsig, (prefetch_size, prefetch_count, a_global),
@@ -2037,3 +2083,7 @@ class Channel(AbstractChannel):
     def _on_basic_ack(self, delivery_tag, multiple):
         for callback in self.events['basic_ack']:
             callback(delivery_tag, multiple)
+
+    def _on_basic_nack(self, delivery_tag, multiple):
+        for callback in self.events['basic_nack']:
+            callback(delivery_tag, multiple)
