@@ -42,6 +42,9 @@ Start from server, version: %d.%d, properties: %s, mechanisms: %s, locales: %s
 __all__ = ['Connection']
 
 AMQP_LOGGER = logging.getLogger('amqp')
+AMQP_HEARTBEAT_LOGGER = logging.getLogger(
+    'amqp.connection.Connection.heartbeat_tick'
+)
 
 #: Default map for :attr:`Connection.library_properties`
 LIBRARY_PROPERTIES = {
@@ -487,14 +490,15 @@ class Connection(AbstractChannel):
         Fetch a Channel object identified by the numeric channel_id, or
         create that object if it doesn't already exist.
         """
-        if self.channels is not None:
-            try:
-                return self.channels[channel_id]
-            except KeyError:
-                channel = self.Channel(self, channel_id, on_open=callback)
-                channel.open()
-                return channel
-        raise RecoverableConnectionError('Connection already closed.')
+        if self.channels is None:
+            raise RecoverableConnectionError('Connection already closed.')
+
+        try:
+            return self.channels[channel_id]
+        except KeyError:
+            channel = self.Channel(self, channel_id, on_open=callback)
+            channel.open()
+            return channel
 
     def is_alive(self):
         raise NotImplementedError('Use AMQP heartbeats')
@@ -510,6 +514,9 @@ class Connection(AbstractChannel):
         return self.on_inbound_frame(frame)
 
     def on_inbound_method(self, channel_id, method_sig, payload, content):
+        if self.channels is None:
+            raise RecoverableConnectionError('Connection already closed')
+
         return self.channels[channel_id].dispatch_method(
             method_sig, payload, content,
         )
@@ -701,8 +708,8 @@ class Connection(AbstractChannel):
         Keyword Arguments:
             rate (int): Previously used, but ignored now.
         """
-        AMQP_LOGGER.debug('heartbeat_tick : for connection %s',
-                          self._connection_id)
+        AMQP_HEARTBEAT_LOGGER.debug('heartbeat_tick : for connection %s',
+                                    self._connection_id)
         if not self.heartbeat:
             return
 
@@ -715,7 +722,7 @@ class Connection(AbstractChannel):
             self.last_heartbeat_received = monotonic()
 
         now = monotonic()
-        AMQP_LOGGER.debug(
+        AMQP_HEARTBEAT_LOGGER.debug(
             'heartbeat_tick : Prev sent/recv: %s/%s, '
             'now - %s/%s, monotonic - %s, '
             'last_heartbeat_sent - %s, heartbeat int. - %s '
@@ -731,7 +738,7 @@ class Connection(AbstractChannel):
 
         # send a heartbeat if it's time to do so
         if now > self.last_heartbeat_sent + self.heartbeat:
-            AMQP_LOGGER.debug(
+            AMQP_HEARTBEAT_LOGGER.debug(
                 'heartbeat_tick: sending heartbeat for connection %s',
                 self._connection_id)
             self.send_heartbeat()
