@@ -1,20 +1,7 @@
 """Exceptions used by amqp."""
 # Copyright (C) 2007-2008 Barry Pederson <bp@barryp.org>
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation; either
-# version 2.1 of the License, or (at your option) any later version.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 from __future__ import absolute_import, unicode_literals
+
 from .five import python_2_unicode_compatible
 from .platform import pack, unpack
 
@@ -28,7 +15,7 @@ __all__ = [
     'ResourceLocked', 'PreconditionFailed', 'FrameError', 'FrameSyntaxError',
     'InvalidCommand', 'ChannelNotOpen', 'UnexpectedFrame', 'ResourceError',
     'NotAllowed', 'AMQPNotImplementedError', 'InternalError',
-
+    'MessageNacked',
     'AMQPDeprecationWarning',
 ]
 
@@ -37,6 +24,10 @@ class AMQPDeprecationWarning(UserWarning):
     """Warning for deprecated things."""
 
 
+class MessageNacked(Exception):
+    """Message was nacked by broker."""
+
+
 @python_2_unicode_compatible
 class AMQPError(Exception):
     """Base class for all AMQP exceptions."""
@@ -58,7 +49,9 @@ class AMQPError(Exception):
     def __str__(self):
         if self.method:
             return '{0.method}: ({0.reply_code}) {0.reply_text}'.format(self)
-        return self.reply_text or '<AMQPError: unknown error>'
+        return self.reply_text or '<{}: unknown error>'.format(
+            type(self).__name__
+        )
 
     @property
     def method(self):
